/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: ServiceManager.java ofabre$
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.bc.ejb.listener;

import java.io.ByteArrayInputStream;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.logging.Level;

import javax.jbi.messaging.MessagingException;
import javax.naming.Context;
import javax.xml.namespace.QName;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import org.ow2.petals.bc.ejb.EjbBC;
import org.ow2.petals.bc.ejb.EjbBCException;
import org.ow2.petals.bc.ejb.model.Params;
import org.ow2.petals.component.framework.listener.AbstractJBIListener;
import org.ow2.petals.component.framework.util.Exchange;
import org.ow2.petals.component.framework.util.Extensions;
import org.ow2.petals.component.framework.util.XMLUtil;

/**
 * 
 * @author ofabre - eBM Websourcing
 * 
 */
public class ServiceManager {

    private AbstractJBIListener listener;

    private EjbBC ejbBC;

    public ServiceManager(AbstractJBIListener listener) {
        super();
        ejbBC = (EjbBC) listener.getComponent();
        this.listener = listener;
    }

    public void onJBIMessage(Exchange exchange) throws EjbBCException {
        listener.getLogger().log(Level.FINE, "Try to call an EJB");

        // Operation to call
        QName operation = exchange.getOperation();

        // Get in message content
        Document requestDocument;
        try {
            requestDocument = exchange.getInMessageContentAsDocument();
        } catch (MessagingException e) {
            throw new EjbBCException("Can't retrieve in message content for exchange : "
                    + exchange.getExchangeId(), e);
        }

        // Retrieve SU extensions
        Extensions extensions = listener.getExtensions();

        // Retrieve Su classloader
        ClassLoader suClassloader = ejbBC.getSuClassLoader(listener.getProvides());

        // Change context classloader to component classloader
        ClassLoader oldCL = Thread.currentThread().getContextClassLoader();
        Thread.currentThread().setContextClassLoader(suClassloader);

        try {
            // Retrieve the EJB Object for this exchange
            Object ejb = ejbBC.getEjb(listener.getProvides());

            // Call ejb
            Document responseDocument = callEjb(ejb, extensions, operation, requestDocument);

            if (responseDocument != null) {
                try {
                    exchange.setOutMessageContent(responseDocument);
                } catch (MessagingException e) {
                    throw new EjbBCException("Can't set out message content for exchange : "
                            + exchange.getExchangeId(), e);
                }
            }
        } finally {
            // reset old classloader
            Thread.currentThread().setContextClassLoader(oldCL);
        }

    }

    private Document callEjb(Object ejb, Extensions extensions, QName operation,
            Document requestDocument) throws EjbBCException {
        // Create an bean representation
        Params params = retrieveEjbMethodModel(requestDocument);
        Object result = null;

        // Search the method to invoke
        Method methodToInvoke;
        try {
            methodToInvoke = ejb.getClass().getMethod(operation.getLocalPart(), params.getTypes());
        } catch (NoSuchMethodException e) {
            throw new EjbBCException("Can't find method '" + operation.getLocalPart()
                    + "' in EJB interface : " + ejb.getClass().getName(), e);
        }

        try {
            result = methodToInvoke.invoke(ejb, params.getParams());
        } catch (SecurityException e) {
            throw new EjbBCException("Error invoking EJB : security error", e);
        } catch (IllegalArgumentException e) {
            throw new EjbBCException("Error invoking EJB, bad method arguments", e);
        } catch (IllegalAccessException e) {
            throw new EjbBCException("Error invoking EJB", e);
        } catch (InvocationTargetException e) {
            // Try to rebind ejb in case of application server connection loss
            Context initialContext = ejbBC.createInitialContext(extensions);
            Object newEjb = ejbBC.retrieveObject(initialContext, extensions);
            ejbBC.addEjb(listener.getProvides(), newEjb);
            // Try a second invocation
            try {
                result = methodToInvoke.invoke(newEjb, params.getParams());
            } catch (Exception e1) {
                throw new EjbBCException("Error invoking EJB", e);
            }
        }
        return createResponseDocument(result);
    }

    private Document createResponseDocument(Object result) throws EjbBCException {
        Document document = null;
        try {
            String stringResult = ejbBC.getXstream(listener.getProvides()).toXML(result);
            DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
            DocumentBuilder documentBuilder = factory.newDocumentBuilder();
            document = documentBuilder.parse(new ByteArrayInputStream(stringResult.getBytes()));
        } catch (Exception e) {
            throw new EjbBCException("Error getting field value", e);
        }
        return document;
    }

    private Params retrieveEjbMethodModel(Document requestDocument) throws EjbBCException {
        Params params = new Params();
        Element element = requestDocument.getDocumentElement();
        NodeList children = element.getChildNodes();
        for (int i = 0; i < children.getLength(); i++) {
            Node node = children.item(i);
            try {
                Object object = ejbBC.getXstream(listener.getProvides()).fromXML(
                        XMLUtil.createStringFromDOMNode(node));
                params.addParam(object);
                params.addParamType(object.getClass());
            } catch (Throwable e) {
                throw new EjbBCException("Can't read input param : " + node.getLocalName(), e);
            }

        }
        return params;
    }

}
