/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: EjbServiceUnitListener.java ofabre$
 * -------------------------------------------------------------------------
 */
package org.ow2.petals.bc.ejb.su;

import java.util.ArrayList;
import java.util.List;

import javax.naming.Context;

import org.ow2.petals.bc.ejb.EjbBC;
import org.ow2.petals.bc.ejb.EjbBCException;
import org.ow2.petals.component.framework.exception.PEtALSCDKException;
import org.ow2.petals.component.framework.su.ServiceUnitListener;
import org.ow2.petals.component.framework.util.ClassLoaderUtil;
import org.ow2.petals.component.framework.util.Extensions;
import org.ow2.petals.jbi.descriptor.JBIDescriptor;
import org.ow2.petals.jbi.descriptor.Provides;
import org.ow2.petals.jbi.descriptor.Services;

import com.thoughtworks.xstream.XStream;

/**
 * 
 * @author ofabre - eBM Websourcing
 *
 */
public class EjbServiceUnitListener implements ServiceUnitListener {

    private static final String XSTREAM_ALIAS = "xstream.alias.";

    private EjbBC ejbBC;

    public EjbServiceUnitListener(EjbBC ejbBC) {
        super();
        this.ejbBC = ejbBC;
    }

    public void onSUDeployed(String serviceUnitName, String suRootPath, JBIDescriptor descriptor)
            throws PEtALSCDKException {
        ClassLoader suClassloader = createSUClassLoader(suRootPath);
        ejbBC.addSuClassLoader(serviceUnitName, suClassloader);
        // Change context classloader to component classloader
        ClassLoader oldCL = Thread.currentThread().getContextClassLoader();
        Thread.currentThread().setContextClassLoader(suClassloader);
        try {
            for (Provides provides : getProvidesList(descriptor)) {
                registerEjb(provides, suRootPath);
                createXstream(provides, suClassloader);
            }
        } catch (EjbBCException e) {
            throw new PEtALSCDKException("Error creating a connection pool for SU: " + suRootPath,
                    e);
        } finally {
            // reset old classloader
            Thread.currentThread().setContextClassLoader(oldCL);
        }

    }

    private void createXstream(Provides provides, ClassLoader suClassLoader)
            throws PEtALSCDKException {
        Extensions extensions = null;
        if (provides.getExtensions() != null) {
            extensions = ejbBC.getExtensions(provides);
        }
        if (extensions != null) {
            XStream stream = new XStream();
            // Add aliases for xml un/marshalling
            for (String param : extensions.getParameters()) {
                if (param.startsWith(XSTREAM_ALIAS)) {
                    Class clazz = loadAliasClass(suClassLoader, extensions.getValue(param));
                    stream.alias(param.substring(XSTREAM_ALIAS.length()), clazz);
                }
            }
            ejbBC.addXstream(provides, stream);
        }
    }

    private Class loadAliasClass(ClassLoader suClassLoader, String value) throws PEtALSCDKException {
        try {
            return suClassLoader.loadClass(value);
        } catch (ClassNotFoundException e) {
            throw new PEtALSCDKException("Can't load xstream alias class : " + value, e);
        }
    }

    private void registerEjb(Provides provides, String suRootPath) throws EjbBCException {
        Extensions extensions = null;
        if (provides.getExtensions() != null) {
            extensions = ejbBC.getExtensions(provides);
        }
        if (extensions != null) {
            // Create an initial context according to given JNDI parameters
            Context initialContext = ejbBC.createInitialContext(extensions);

            // Retrieve the EJB Object matching the given name
            Object ejb = ejbBC.retrieveObject(initialContext, extensions);

            // Register ejb
            ejbBC.addEjb(provides, ejb);
        }
    }

    private ClassLoader createSUClassLoader(String suRootPath) {
        return ClassLoaderUtil.createClassLoader(suRootPath, this.getClass().getClassLoader());
    }

    public void onSUStarted(String arg0) throws PEtALSCDKException {
        // TODO Auto-generated method stub

    }

    public void onSUStopped(String arg0) throws PEtALSCDKException {
        // TODO Auto-generated method stub

    }

    public void onSUUndeployed(String serviceUnitName, String suRootPath, JBIDescriptor descriptor)
            throws PEtALSCDKException {
        for (Provides provides : getProvidesList(descriptor)) {
            ejbBC.removeEjb(provides);
            ejbBC.removeXstream(provides);
        }
        ejbBC.removeClassLoader(serviceUnitName);
    }

    private List<Provides> getProvidesList(JBIDescriptor descriptor) {
        List<Provides> providesList = new ArrayList<Provides>();
        Services services = descriptor.getServices();
        if (services != null) {
            providesList = services.getProvides();
        }
        return providesList;
    }

}
